dnl ######################################################################
dnl
dnl File:	txphysics.m4
dnl
dnl Purpose:	Determine the locations of the txphysic component
dnl		includes and libraries. Componenets are nsecc and 
dnl		txcrange.
dnl 		Allows also for the installation of the ML library. 
dnl 
dnl Version: $Id: txphysics.m4,v 1.2 2005/08/09 22:28:43 cary Exp $
dnl
dnl Tech-X configure system
dnl
dnl ######################################################################


dnl ######################################################################
dnl
dnl Allow the user to specify an overall txphysics directory.  If specified,
dnl we look for include and lib under this.
dnl
dnl ######################################################################

AC_ARG_WITH(txphysics-dir,[  --with-txphysics-dir=<location of txphysics installation> ],TXPHYSICS_DIR="$withval",TXPHYSICS_DIR="")

dnl ######################################################################
dnl
dnl Find nsecc and txcrange includes - looking in include location if present,
dnl otherwise in dir/include if present, otherwise in default locations,
dnl first parallel, then serial.
dnl
dnl ######################################################################

AC_ARG_WITH(txphysics-incdir,[  --with-txphysics-incdir=<location of txphysics includes> ],
TXPHYSICS_INCDIR="$withval",TXPHYSICS_INCDIR="")
if test -n "$TXPHYSICS_INCDIR"; then
  TXPHYSICS_INCPATH=$TXPHYSICS_INCDIR
elif test -n "$TXPHYSICS_DIR"; then
  TXPHYSICS_INCPATH=$TXPHYSICS_DIR/include
else
  TXPHYSICS_INCPATH="/usr/local/txphysics/include"
fi

if test -n "$TXPHYSICS_INCPATH"; then 
  AC_PATH_PROGS(NSECC_H, nsecc.h, "", $TXPHYSICS_INCPATH)
  if test -z "$NSECC_H"; then
    AC_MSG_WARN(nsecc.h not found in $TXPHYSICS_INCPATH.  Set with --with-txphysics-incdir=)
    TXPHYSICS_INC=" "
  else
    NSECC_INCDIR=`dirname $NSECC_H`
    AC_SUBST(NSECC_INCDIR)
    NSECC_INC="-I$NSECC_INCDIR"

    AC_SUBST(NSECC_INC)
    NSECC_DIR=`dirname $NSECC_INCDIR`
  fi
  AC_PATH_PROGS(TXCRANGE_H, txcrange.h, "", $TXPHYSICS_INCPATH)
  if test -z "$TXCRANGE_H"; then
    AC_MSG_WARN(txcrange.h not found in $TXPHYSICS_INCPATH.  Set with --with-txphysics-incdir=)
    TXPHYSICS_INC=" "
  else
    TXCRANGE_INCDIR=`dirname $TXCRANGE_H`
    AC_SUBST(TXCRANGE_INCDIR)
    TXCRANGE_INC="-I$TXCRANGE_INCDIR"

    AC_SUBST(TXCRANGE_INC)
    TXCRANGE_DIR=`dirname $TXCRANGE_INCDIR`
  fi
fi

dnl ######################################################################
dnl
dnl Find nsecc and txcrange libraries
dnl
dnl ######################################################################

AC_ARG_WITH(txphysics-libdir,[  --with-txphysics-libdir=<location of txphysics libraries> ],
TXPHYSICS_LIBDIR="$withval",TXPHYSICS_LIBDIR="")

if test -n "$TXPHYSICS_LIBDIR"; then
  TXPHYSICS_LIBPATH=$TXPHYSICS_LIBDIR
elif test -n "$TXPHYSICS_DIR"; then
  TXPHYSICS_LIBPATH=$TXPHYSICS_DIR/lib
else
  TXPHYSICS_LIBPATH=`dirname $TXCRANGE_INCDIR`/lib
fi

if test -n "$TXPHYSICS_LIBPATH"; then
  AC_PATH_PROGS(LIBNSECC_A, libnsecc.a, "", $TXPHYSICS_LIBPATH)
  if test -z "$LIBNSECC_A"; then
    AC_MSG_WARN(libnsecc.a not found.  Set with --with-txphysics-libdir=)
    NSECC_LIBS=""
  else
    NSECC_LIBDIR=`dirname $LIBNSECC_A`
    AC_SUBST(NSECC_LIBDIR)
    NSECC_LIBS="-L$NSECC_LIBDIR -lnsecc $FLIBS"' $(RPATH_FLAG)'"$NSECC_LIBDIR"
  fi
  AC_SUBST(NSECC_LIBS)

  AC_PATH_PROGS(LIBTXCRANGE_A, libtxcrange.a, "", $TXPHYSICS_LIBPATH)
  if test -z "$LIBTXCRANGE_A"; then
    AC_MSG_WARN(libtxcrange.a not found.  Set with --with-txphysics-libdir=)
    TXCRANGE_LIBS=""
  else
    TXCRANGE_LIBDIR=`dirname $LIBTXCRANGE_A`
    AC_SUBST(TXCRANGE_LIBDIR)
    TXCRANGE_LIBS="-L$TXCRANGE_LIBDIR -ltxcrange $FLIBS"' $(RPATH_FLAG)'"$TXCRANGE_LIBDIR"
  fi
  AC_SUBST(TXCRANGE_LIBS)
fi

dnl ######################################################################
dnl
dnl Defines for whether nsecc and txcrange found
dnl
dnl ######################################################################


if test -n "$NSECC_LIBS"; then
  AC_DEFINE(HAVE_NSECC)
fi
if test -n "$TXCRANGE_LIBS"; then
  AC_DEFINE(HAVE_TXCRANGE)
fi


